#include "tclSynthesizer.h"

extern hub::Hub theHub;

int tclSynthesizer( ClientData clientData, Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[] ){
	hub::SrcLocation src( &theHub );
	hub::DestLocation dest( &theHub );
	Tcl_Obj* result;

	if ( objc != 6 ) {
		if( objc == 2 ){
			return	tclGetAlgorithms( clientData, interp, objc, objv );
		}
		else{
			Tcl_WrongNumArgs(interp, 1, objv, "synthesize src dest stateMinimizationAlgorithm encodingAlgorithm realizationAlgorithm\nsynthesize getStateMinimizationAlgorithms\nsynthesize getEncodingAlgorithms\nsynthesize getRealizationAlgorithms");
			return TCL_ERROR;
		}
	}
	infStream << "Synthesis start" << std::endl;
	try {
		src = Tcl_GetString ( objv [1] );
		dest = Tcl_GetString ( objv [2] );
		src.Find();
		//Arch2ArchProc( src, dest, objv, 1, 2 );
		StateMinimizationAlgorithmIdentificator stateMinimizationAlgorithm( Tcl_GetString (objv [3]) );
		EncodingAlgorithmIdentificator encodingAlgorithm( Tcl_GetString (objv [4]) );
		RealizationAlgorithmIdentificator realizationAlgorithm( Tcl_GetString (objv [5]) );

		Synthesizer synthesizer( &theHub );

		synthesizer( src, dest, stateMinimizationAlgorithm, encodingAlgorithm, realizationAlgorithm );

	} catch( std::runtime_error e ){ 
		std::string msg = e.what();
		result = Tcl_GetObjResult (interp);
		Tcl_SetStringObj(result, msg.c_str(), msg.length());
		return TCL_ERROR;

	} catch( std :: logic_error e ){
		std::string msg = e.what();
		result = Tcl_GetObjResult (interp);
		Tcl_SetStringObj(result, msg.c_str(), msg.length());
		return TCL_ERROR;
	}

	infStream << "Synthesis done" << std::endl;
	
	result = Tcl_NewStringObj( dest.Actual().c_str(), dest.Actual().length() );

	Tcl_SetObjResult ( interp, result );

	return TCL_OK;
}

int tclGetAlgorithms( ClientData, Tcl_Interp* interp, int objc, Tcl_Obj* CONST objv[] ){
	
	if( objc != 2 ){
		Tcl_WrongNumArgs(interp, 1, objv, "wrong command argument");
		return TCL_ERROR;
	}
	Tcl_Obj* result = Tcl_NewListObj( 0, 0 );

	char* command = Tcl_GetString( objv[1] );

	if ( strcmp( command, "getStateMinimizationAlgorithms" ) == 0 ) {
		for( int i=0; (StateMinimizationAlgorithmID)STATE_MINIMIZATION_ALGORITHMS_CHOICE[i] != lastM; i++ ){
			
			Tcl_Obj* row[2];
			row[0] = Tcl_NewStringObj( std::string(STATE_MINIMIZATION_ALGORITHMS_CHOICE[i]).c_str(), -1 );
			row[1] = Tcl_NewStringObj( STATE_MINIMIZATION_ALGORITHMS_CHOICE[i].getName().c_str(), -1 );

			Tcl_Obj* rResult = Tcl_NewListObj( 1, row );	//TODO descriptions
			Tcl_ListObjAppendElement( interp, result, rResult );
		}
	}
	else if ( strcmp( command, "getEncodingAlgorithms" ) == 0 ) {
		for( int i=0; (EncodingAlgorithmID)ENCODING_ALGORITHMS_CHOICE[i] != lastE; i++ ){
			
			Tcl_Obj* row[2];
			row[0] = Tcl_NewStringObj( std::string(ENCODING_ALGORITHMS_CHOICE[i]).c_str(), -1 );
			row[1] = Tcl_NewStringObj( ENCODING_ALGORITHMS_CHOICE[i].getName().c_str(), -1 );

			Tcl_Obj* rResult = Tcl_NewListObj( 1, row );
			Tcl_ListObjAppendElement( interp, result, rResult );
		}
	}
	else if ( strcmp( command, "getRealizationAlgorithms" ) == 0 ) {
		for( int i=0; (RealizationAlgorithmID)REALIZATION_ALGORITHMS_CHOICE[i] != lastR; i++ ){
			
			Tcl_Obj* row[2];
			row[0] = Tcl_NewStringObj( std::string(REALIZATION_ALGORITHMS_CHOICE[i]).c_str(), -1 );
			row[1] = Tcl_NewStringObj( REALIZATION_ALGORITHMS_CHOICE[i].getName().c_str(), -1 );

			Tcl_Obj* rResult = Tcl_NewListObj( 1, row );
			Tcl_ListObjAppendElement( interp, result, rResult );
		}
	}
	else{
		Tcl_WrongNumArgs(interp, 1, objv, "wrong command argument");
		return TCL_ERROR;
	}

	Tcl_SetObjResult ( interp, result );

	return TCL_OK;
}
