//class implementing REALIZATION module
//	-module translating encoded FSM into structured gate level description(as far as HUB structures allows)
//author: Petr Kraus
//////////////////////////////////////////////////////////////////////////

#ifndef REALIZER_H
#define REALIZER_H

#pragma warning( push )
#pragma warning( disable : 4996 4100 )	//usage of old stricmp and unreferenced parameter e in hub
#include "hub/hub.h"
#include "hub/location.h"
#include "hub/structural.h"
#include "hub/symfunc.h"
#include "hub/fsm.h"
#include "hub/err.h"
#pragma warning( pop )

#include <list>
#include <vector>
#include <string>
#include <sstream>
#include "Algorithms.h"

class Realizer
{
	hub::Hub* theHub;

	static bool isBinary( std::string str );
	static std::string i2Str( unsigned ui );
	static std::string i2Str( int i );

	static std::list< hub::StateTableRow >& createTransitionTable( const hub::FsmDescr* fsm );

	static hub::Entity* createFuncEntity( std::string name, hub::SymFunc::Func func, unsigned inputCount, hub::Library* destLibrary );
	//only D ff based 5 inputs elements
	static hub::Instance* createStateMemory( std::string name, hub::BehavAgent* memoryElement, unsigned inputCount, hub::Library* destLibrary, hub::StructArch* destArch );
	
	static hub::Net* createMintermNet(	unsigned j, unsigned stateCount, unsigned inputCount, std::list< hub::StateTableRow >::iterator it, hub::StructArch* decoderArchImpl,
										hub::Entity* andEntity, hub::Entity* invEntity, hub::Instance* vccInstance );
	static hub::Instance* createTransitionsDecoder(	std::string name, std::list< hub::StateTableRow >& transitionsTable, hub::Library* destLibrary, hub::StructArch* destArch,
													hub::Entity* andEntity, hub::Entity* orEntity, hub::Entity* invEntity, hub::Entity* vccEntity, hub::Entity* gndEntity );


	//////////////////////////////////////////////
	//add your new algorithm implementation here//
	//////////////////////////////////////////////

	//Realization algorithm prototype
	void doNaiveRTLDFFbased( const hub::FsmDescr* srcFSMDescr, hub::StructArch* destArchImpl );

public:
	Realizer( hub::Hub* theHub );
	~Realizer();

	//does your realization job. runs selected algorithm
	//There are no input checks - input is checked in Synthesizer!!! If you plan to use this module separately, implement bad user input exceptions
	void operator() ( hub::SrcLocation& src, hub::DestLocation& dest, RealizationAlgorithmID algorithmID );
};

#endif
