//class describing and identifying algorithm
//	|-> StateMinimizationAlgorithmIdentificator
//	|-> EncodingAlgorithmIdentificator
//	|-> RealizationAlgorithmIdentificator
//	-used for algorithm identification and provides algorithm string description for UIs
//	-can be extended to store any information about algorithm
//author: Petr Kraus
//////////////////////////////////////////////////////////////////////////

#ifndef ALGORITHMIDENTIFICATOR_H
#define ALGORITHMIDENTIFICATOR_H

#pragma warning( push )
#pragma warning( disable: 4100 )	//usage of old stricmp and unreferenced parameter e in hub
#include "hub/err.h"
#pragma warning( pop )

#include <cstring>
#include "Algorithms.h"


class AlgorithmIdentificator{
	std::string uniqueIdentificator;	//identification name
	std::string name;	//full name
	
protected:
	AlgorithmIdentificator( std::string name, std::string uniqueIdentificator );

public:
	//abstract class
	~AlgorithmIdentificator();
	
	//string conversion
	operator std::string() const;	//get identificator
	std::string getName() const;
};

//	stateMinimizationAlgorithmIdentificator
//////////////////////////////////////////////////////////////////////////////////////////////
class StateMinimizationAlgorithmIdentificator : public AlgorithmIdentificator{
	enum StateMinimizationAlgorithmID ID;
	
public:
	StateMinimizationAlgorithmIdentificator( std::string name, std::string uniqueIdentificator, enum StateMinimizationAlgorithmID ID );
	StateMinimizationAlgorithmIdentificator( std::string uniqueIdentificator );	//finds string in choice list
	~StateMinimizationAlgorithmIdentificator();

	operator StateMinimizationAlgorithmID() const;

};

//	EncodingAlgorithmIdentificator
//////////////////////////////////////////////////////////////////////////////////////////////
class EncodingAlgorithmIdentificator : public AlgorithmIdentificator{
	enum EncodingAlgorithmID ID;

public:
	EncodingAlgorithmIdentificator( std::string name, std::string uniqueIdentificator, enum EncodingAlgorithmID ID );
	EncodingAlgorithmIdentificator( std::string uniqueIdentificator );	//finds string in choice list
	~EncodingAlgorithmIdentificator();

	operator EncodingAlgorithmID() const;

};

//	RealizationAlgorithmIdentificator
//////////////////////////////////////////////////////////////////////////////////////////////
class RealizationAlgorithmIdentificator : public AlgorithmIdentificator{
	enum RealizationAlgorithmID ID;

public:
	RealizationAlgorithmIdentificator( std::string name, std::string uniqueIdentificator, enum RealizationAlgorithmID ID );
	RealizationAlgorithmIdentificator( std::string uniqueIdentificator );	//finds string in choice list
	~RealizationAlgorithmIdentificator();

	operator RealizationAlgorithmID() const;

};

ALGORITHM_LIST	//viz algorithms.h

#endif
